/* -------------------------------------------------------- */
/* cdfread.c                                                */
/* functions to read CDF files                              */
/* -------------------------------------------------------- */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "copyrght.h"
#include "cdfread.h"

/* pointers for pdf1 files */
static FILE * fbitmap;                   /* selected bitmap library */

/* --------------------------------- */
/* Open a bitmap library (.bit) file */
/* --------------------------------- */
INTEGER4 MSFLAG PS_OPENBITLIB(
		   const CHAR * filename,  /* name of file */
#ifdef _MSC_VER
		      int FORlen1, /* dummy FORTRAN string length */
#endif
		   const CHAR * diskpath,  /* location of disk files */
#ifdef _MSC_VER
		      int FORlen2, /* dummy FORTRAN string length */
#endif
		   const CHAR * cdrompath
#ifdef _MSC_VER
		      , int FORlen3 /* dummy FORTRAN string length */
#endif
		   ) /* location of cdrom files */

{
	char tempname[128];

	/* first try to open the file on disk */
	strcpy(tempname,diskpath);
	strcat(tempname,filename);
	fbitmap = fopen(tempname, "rb");
	if (fbitmap != NULL)  return 0;

	/* no luck, now try to open the file on cdrom */
	strcpy(tempname,cdrompath);
	strcat(tempname,filename);
	fbitmap = fopen(tempname, "rb");
	if (fbitmap != NULL)  return 0;
	return -1;
}

/* ----------------------------- */
/* Close the bitmap library file */
/* ----------------------------- */
void MSFLAG PS_CLOSEBITLIB(void)
{
	if (fbitmap != NULL) fclose(fbitmap);
	fbitmap = NULL;
}				


/* ---------------------------------------- */
/* Read a record from a bitmap library file */
/* ---------------------------------------- */
INTEGER4 MSFLAG PS_READBITLIB(
		  INTEGER4 *recpointer, /* sequence # to be read */
		  INTEGER4 *table,      /* bitmap array */
		  INTEGER4 *nentries)   /* number of entries in database */
{
  int i2;
  long bytepointer;
  size_t nwords; 
  
  nwords = (31+*nentries)/32;
  bytepointer = 4 * nwords * (-1 + *recpointer); /* (4*nwords bytes/record) */

  i2 = fseek(fbitmap, bytepointer, SEEK_SET);
  if (i2<0) return -1L;

  i2 = fread(table, 4, nwords, fbitmap);
  if (i2<nwords) return -1L;
  return 0L;
}

/* save the bitmap on file */
INTEGER4 MSFLAG PS_WRITEMAP(
		   const CHAR * filename,  /* location of disk files */
#ifdef _MSC_VER
		      int FORlen, /* dummy FORTRAN string length */
#endif
		   INTEGER4 *nentries,         /* # of JCPDS-ICDD entries */
		   INTEGER4 *bitmap,	   /* pointer to bitmap array */
		   INTEGER4 *histnum,	   /* number of history records*/
		   CHAR     *history,	   /* array of history records*/
#ifdef _MSC_VER
		      int FORlen1, /* dummy FORTRAN string length */
#endif
		   INTEGER4 *histlen)	   /* length of history array in bytes*/
{
  FILE * filep;
  size_t nwords;
  int i2;

/* open the file */
  filep = fopen(filename, "wb");
  if (filep == NULL) {
    /*fclose(filep); */
    return -1L;
  }

/* Write a record to the pdf1 index file */
  i2 = fwrite(nentries, 4, 1, filep);
  if (i2<1) {
    fclose(filep); 
    return -2L;
  }

/* Write a record to the pdf1 index file */
  nwords = (31+*nentries)/32;
  i2 = fwrite(bitmap, 4, nwords, filep);
  if (i2<nwords) {
    fclose(filep); 
    return -3L;
  }

/* Write a record to the pdf1 index file */
  i2 = fwrite(histnum, 4, 1, filep);
  if (i2<1) {
    fclose(filep); 
    return -4L;
  }

/* Write a record to the pdf1 index file */
  nwords = (31+*nentries)/32;
  i2 = fwrite(history, *histlen, *histnum, filep);
  if (i2<*histnum) {
    fclose(filep); 
    return -5L;
  }

  fclose(filep); 
  return 0;
}			

/* read the bitmap from a file */
INTEGER4 MSFLAG PS_READMAP(
		   const CHAR * filename,  /* location of disk files */
#ifdef _MSC_VER
		      int FORlen, /* dummy FORTRAN string length */
#endif
		   INTEGER4 *nentries,         /* # of JCPDS-ICDD entries */
		   INTEGER4 *bitmap,	   /* pointer to bitmap array */
		   INTEGER4 *histnum,	   /* number of history records*/
		   CHAR     *history,	   /* array of history records*/
#ifdef _MSC_VER
		      int FORlen1, /* dummy FORTRAN string length */
#endif
		   INTEGER4 *histlen)	   /* length of history array in bytes*/
{
  FILE * filep;
  size_t nwords;
  int i2;

/* open the file */
  filep = fopen(filename, "rb");
  if (filep == NULL) {
    /* fclose(filep); */
    return -1L;
  }

/* Read a record to the pdf1 index file */
  i2 = fread(nentries, 4, 1, filep);
  if (i2<1) {
    fclose(filep); 
    return -2L;
  }

/* Read a record to the pdf1 index file */
  nwords = (31+*nentries)/32;
  i2 = fread(bitmap, 4, nwords, filep);
  if (i2<nwords) {
    fclose(filep); 
    return -3L;
  }

/* Read a record to the pdf1 index file */
  i2 = fread(histnum, 4, 1, filep);
  if (i2<1) {
    fclose(filep); 
    return -4L;
  }

/* Read a record to the pdf1 index file */
  nwords = (31+*nentries)/32;
  i2 = fread(history, *histlen, *histnum, filep);
  if (i2<*histnum) {
    fclose(filep); 
    return -5L;
  }

  fclose(filep); 
  return 0;
}			
