C========================================================================
C========================================================================
C
C Program to search the NIST Crystal Data Database by reduced cell 
C 
C
C========================================================================
C input line 1: a,b,c,alpha,beta,gamma
C       line 2a: error on a,b,c (A)
C       line 2b: error on alpha,beta,gamma (degrees)
C========================================================================
      IMPLICIT NONE

      include 'size.par'
      include 'map.cmn'
      include 'cdf.cmn'
      include 'cdfread.def'
      include 'bitops.def'

      INTEGER IARGC,STATUS,mode
      INTEGER I,I1,I2,J,NARG,LENCH,k
      integer psl_readbitmap
      integer psl_savebitmap
      CHARACTER*256 INFILE,OUTFILE
      CHARACTER*100 trace
      REAL CELL(6),angerr,lnerr,lgmin,lgmax,x1,x2
      integer len2ind,ang2ind
      logical angsea


      INFILE = ' '
      OUTFILE = ' '
C initialize the database
      call ps_CDinit(STATUS)
      call psl_mapinit(1)
      IF (STATUS .ne. 0) STOP 'Error in CDINIT'
      NARG = IARGC()
      IF (NARG .GE. 1) THEN
         CALL GETARG(1,OUTFILE)
      ENDIF
      IF (NARG .GE. 2) THEN
         CALL GETARG(2,INFILE)
      ENDIF
      IF (outfile .eq. ' ') THEN
         write (*,*) 'No output file'
         stop 1
      ENDIF
      IF (infile .ne. ' ') THEN
         STATUS = psl_readbitmap(infile,-1)
         if (STATUS .ne. 0) stop 1
         WRITE (*,*) 'Reading: ',infile(:lench(infile))
      ENDIF
      I1 = 0
      I2 = 0
      do i=1,6
         cell(i) = 0.0
      enddo
      lnerr = 0.
      angerr = 0.
C cell
      READ (*,'(A)',end=1) trace
      read (trace,*,end=1) cell
C errors
 1    READ (*,'(A)',end=2) trace
      read (trace,*,end=2) lnerr,angerr

 2    nph1 = TBLSET(TABLE1,N_Cards)
C search on cell lengths
      do i=1,3
         CALL ps_OpenCell(i,STATUS)
         IF (status .ne. 0) THEN
            WRITE (*,*) 'Unable to open cell bitmap file #',i
            STOP 1
         ENDIF
C	convert length to index numbers 
         i1 = Len2ind(cell(i) - lnerr)
         i2 = Len2ind(cell(i) + lnerr)
         nph2 = TBLCLR(TABLE2,N_Cards)
         DO j=i1,i2
            k = ps_readbitlib(j, table3, N_cards)
            CALL TBLOR(TABLE2,TABLE3,TABLE2,N_Cards)
         enddo
         CALL TBLAND(TABLE1,TABLE2,TABLE1,N_Cards)
         call ps_closebitlib()
      enddo
C search on angles?
      angsea = .false.
      if (cell(4) .ne. 0 .and. cell(5) .ne. 0 .and.
     $     cell(6) .ne. 0 .and. angerr .gt. 0 .and.
     $     angerr .lt. 60) then
         angsea = .true.
         do i=4,6
            CALL ps_OpenCell(i,STATUS)
            IF (status .ne. 0) THEN
               WRITE (*,*) 'Unable to open cell bitmap file #',i
               STOP 1
            ENDIF
C     convert angles to index numbers 
            i1 = ang2ind(cell(i) - angerr)
            i2 = ang2ind(cell(i) + angerr)
            if (i1 .gt. 1 .or. i2 .lt. MAX_ANG) then
               nph2 = TBLCLR(TABLE2,N_Cards)
               DO j=i1,i2
                  k = ps_readbitlib(j, table3, N_cards)
                  CALL TBLOR(TABLE2,TABLE3,TABLE2,N_Cards)
               enddo
               CALL TBLAND(TABLE1,TABLE2,TABLE1,N_Cards)
            endif
            call ps_closebitlib()
         enddo
      endif
C========================================================================
C now setup the history entry
C========================================================================
      trace = 'Reduced cell search'
      if (lnerr .lt. 10.) then
         write(trace(lench(trace)+2:),'(A,f5.3)')
     $        'cell length err = ',lnerr
      else
         write(trace(lench(trace)+2:),'(A,f5.2)')
     $        'cell length err = ',lnerr
      endif
      if (angsea) then
         if (angerr .lt. 10.) then
            write(trace(lench(trace)+1:),'(A,f5.3)')
     $           ', cell angle err = ',angerr
         else
            write(trace(lench(trace)+1:),'(A,f5.2)')
     $           ', cell angle err = ',angerr
         endif
      endif
      CALL TBLLOGIC(0, trace)
C add additional info to history
      do i=1,3
         if (i .eq. 1) then
            trace = ' ...a='
         elseif (i .eq. 2) then
            trace = ' ...b='
         else
            trace = ' ...c='
         endif
         if (cell(i) .lt. 10.) then
            write(trace(lench(trace)+1:),'(f5.3)') cell(i)
         elseif (cell(i) .lt. 100.) then
            write(trace(lench(trace)+1:),'(f5.2)') cell(i)
         else
            write(trace(lench(trace)+1:),'(f6.2)') cell(i)
         endif
C	convert length to index numbers 
         i1 = Len2ind(cell(i) - lnerr)
         i2 = Len2ind(cell(i) + lnerr)
         Call ind2Len(i1,x1,lgmin,x2)
         Call ind2Len(i2,x1,x2,lgmax)
         trace(lench(trace)+2:) = '(actual'
         if (lgmin .lt. 10.) then
            write(trace(lench(trace)+2:),'(f5.3)') lgmin
         elseif (lgmin .lt. 100.) then
            write(trace(lench(trace)+2:),'(f5.2)') lgmin
         else
            write(trace(lench(trace)+2:),'(f6.2)') lgmin
         endif
         trace(lench(trace)+1:) = '-'
         if (lgmax .lt. 10.) then
            write(trace(lench(trace)+1:),'(f5.3)') lgmax
         elseif (lgmax .lt. 100.) then
            write(trace(lench(trace)+1:),'(f5.2)') lgmax
         else
            write(trace(lench(trace)+1:),'(f6.2)') lgmax
         endif
         trace(lench(trace)+1:) = ')'
         if (cnum .lt. cnum_max) THEN
            cnum = cnum + 1
            context(cnum) = trace
         endif
      enddo
      if (angsea) then 
         do i=4,6
            i1 = ang2ind(cell(i) - angerr)
            i2 = ang2ind(cell(i) + angerr)
            if (i1 .gt. 1 .or. i2 .lt. MAX_ANG) then
               if (i .eq. 4) then
                  trace = ' ...alpha='
               elseif (i .eq. 5) then
                  trace = ' ...beta='
               else
                  trace = ' ...gamma='
               endif
               if (cell(i) .lt. 100.) then
                  write(trace(lench(trace)+1:),'(f5.2)') cell(i)
               else
                  write(trace(lench(trace)+1:),'(f6.2)') cell(i)
               endif
C     convert length to index numbers 
               Call ind2ang(i1,x1,lgmin,x2)
               Call ind2ang(i2,x1,x2,lgmax)
               trace(lench(trace)+2:) = '(actual'
               if (lgmin .lt. 100.) then
                  write(trace(lench(trace)+2:),'(f5.2)') lgmin
               else
                  write(trace(lench(trace)+2:),'(f6.2)') lgmin
               endif
               trace(lench(trace)+1:) = '-'
               if (lgmax .lt. 100.) then
                  write(trace(lench(trace)+1:),'(f5.2)') lgmax
               else
                  write(trace(lench(trace)+1:),'(f6.2)') lgmax
               endif
               trace(lench(trace)+1:) = ')'
               if (cnum .lt. cnum_max) THEN
                  cnum = cnum + 1
                  context(cnum) = trace
               endif
            endif
         enddo
      endif
      nphase = TBLCOUNT(TABLE,N_Cards)
      WRITE(*,*) 'Found ',nphase,' entries'
      STATUS = psl_savebitmap(outfile)
      if (STATUS .ne. 0) stop 1
      WRITE (*,*) 'Writing ',outfile(:lench(outfile))
      STOP
      END

