C========================================================================
C========================================================================
C
C Program to search the NIST Crystal Data Database by elements
C
C========================================================================
C input line 1: list of required elements by number
C       line 2: opt mode = 0 = all optional
C                        = 1 = specific elements allowed
C       line 3: list of allowed elements by number (mode = 1)
C
C note that element numbers of -1 to -18 are 1a,2a,...AC
C========================================================================
      IMPLICIT NONE

      include 'size.par'
      include 'map.cmn'
      include 'cdf.cmn'
      include 'cdfread.def'
      include 'bitops.def'

      INTEGER IARGC,STATUS,mode
      INTEGER I,I1,I2,J,NARG,LENCH,k
      INTEGER*2 ELEMS(104),AELEMS(104)
      integer psl_readbitmap
      integer psl_savebitmap
      CHARACTER*256 INFILE,OUTFILE
      CHARACTER*100 trace
C elements used in groups
      integer   specsym(16,18) 
      LOGICAL allowed(104)

	CHARACTER*2	ELEMLIST(104)
      DATA ELEMLIST/'H ','He','Li','Be','B ','C ','N ','O ','F ','Ne',
     1'Na','Mg','Al','Si','P ','S ','Cl','Ar','K ','Ca','Sc','Ti','V ',
     2'Cr','Mn','Fe','Co','Ni','Cu','Zn','Ga','Ge','As','Se','Br','Kr',
     3'Rb','Sr','Y ','Zr','Nb','Mo','Tc','Ru','Rh','Pd','Ag','Cd','In',
     4'Sn','Sb','Te','I ','Xe','Cs','Ba','La','Ce','Pr','Nd','Pm','Sm','
     5Eu','Gd','Tb','Dy','Ho','Er','Tm','Yb','Lu','Hf','Ta','W ','Re','O
     6s','Ir','Pt','Au','Hg','Tl','Pb','Bi','Po','At','Rn','Fr','Ra',
     7'Ac','Th','Pa','U ','Np','Pu','Am','Cm','Bk','Cf','Es','Fm','Md',
     8 'No','Lw','Ln'/
C define elements used in groups
C element "1" symbol 1A - period 1 (Li,Na,K,Rb,Cs,Fr)
	DATA specsym/
     5  3,11,19,37,55,87, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
C element "2" symbol 2A - period 2 (Be,Mg,Ca,Sr,Ba,Ra)
     6  4,12,20,38,56,88, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
C element "3" symbol 3A - period 3 (B,Al,Ga,In,Tl)
     7  5,13,31,49,81, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
C element "4" symbol 4A - period 4 (C,Si,Ge,Sn,Pb)
     8  6,14,32,50,82, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
C element "5" symbol 5A - period 5 (N,P,As,Sb,Bi)
     9  7,15,33,51,83, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
C element "6" symbol 6A - period 6 (O,S,Se,Te,Po)
     1  8,16,34,52,84, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
C element "7" symbol 7A - period 7 (F,Cl,Br,I,At)
     1  9,17,35,53,85, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
C element "8" symbol 8A - period 8 (He,Ne,Ar,Kr,Xe,Rn)
     2 10,18,36,54,86, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
C element "9" symbol 1B - transition metals block 9 (Cu,Ag,Au)
     3 29,47,79, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
C element "10" symbol 2B - transition metals block 10 (Zn,Cd,Hg)
     4 30,48,80, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
C element "11" symbol 3B - transition metals block 1 (Sc,Y,La,Ac)
     5 21,39,57,89, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
C element "12" symbol 4B - transition metals block 2 (Ti,Zr,Hf)
     6 22,40,72, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
C element "13" symbol 5B - transition metals block 3 (V,Nb,Ta)
     7 23,41,73, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
C element "14" symbol 6B - transition metals block 4 (Cr,Mo,W)
     8 24,42,74, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
C element "15" symbol 7B - transition metals block 5 (Mn,Tc,Re)
     9 25,43,75, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
C element "16" symbol 8B - transition metals block 6-8 (Fe-Ni,Ru-Pd,Os-Pt)
     1 26,27,28,44,45,46,76,77,78, 0, 0, 0, 0, 0, 0, 0,
C element "17" symbol LN - Lanthanides (La - Lu)
     4 57,58,59,60,61,62,63,64,65,66,67,68,69,70,71,104,
C element "18" symbol AD - Actinides (Ac-Lr)
     1 89,90,91,92,93,94,95,96,97,98,99,100,101,102,103,0/

      INFILE = ' '
      OUTFILE = ' '
C initialize the database
      call ps_CDinit(STATUS)
      call psl_mapinit(1)
      IF (STATUS .ne. 0) STOP 'Error in CDINIT'
      NARG = IARGC()
      IF (NARG .GE. 1) THEN
         CALL GETARG(1,OUTFILE)
      ENDIF
      IF (NARG .GE. 2) THEN
         CALL GETARG(2,INFILE)
      ENDIF
      IF (outfile .eq. ' ') THEN
         write (*,*) 'No output file'
         stop 1
      ENDIF
      IF (infile .ne. ' ') THEN
         STATUS = psl_readbitmap(infile,-1)
         if (STATUS .ne. 0) stop 1
         WRITE (*,*) 'Reading: ',infile(:lench(infile))
      ENDIF
      I1 = 0
      I2 = 0
      DO I=1,104
         elems(I) = 0
         aelems(I) = 0
      enddo
C required elements
      READ (*,'(A)',end=1) trace
      read (trace,*,end=1) (elems(I),I=1,104)
C mode
 1    READ (*,'(A)',end=2) trace
      read (trace,*,end=2) mode
 2    if (mode .ne. 0) then
         do i=1,104
            allowed(i) = .false.
         enddo
C allowed elements
         READ (*,'(A)',end=3) trace
         read (trace,*,end=3) (aelems(I),I=1,104)
      endif
 3    CALL ps_openchem(STATUS)
      IF (status .ne. 0) THEN
	 WRITE (*,*) 'Unable to open chemistry bitmap file'
	 STOP 1
      ENDIF

      nph1 = TBLSET(TABLE1,N_Cards)
      DO I=1,104
         I1 = ELEMS(I)
         if (I1 .ge. 1 .and. I1 .le. 104) then
            allowed(i1) = .true.
            k = ps_readbitlib(i1, table2, N_cards)
            IF (k .ne. 0) THEN
               WRITE (*,*) 'Error reading bitmap file'
               STOP 1
            ENDIF
            CALL TBLAND(TABLE1,TABLE2,TABLE1,N_Cards)
C     if a lanthanide is present, LN must be allowed
            IF (I1 .ge. 57 .and. I1 .le. 71) allowed(104) = .true.
         elseif (I1 .le.-1 .and. I1 .ge. -18) then
            nph1 = TBLCLR(TABLE2,N_Cards)
            I1 = -I1
            do j=1,16
               i2 = specsym(j,I1)
               if (i2 .gt. 0) THEN
                  allowed(i2) = .true.
                  k = ps_readbitlib(i2, table3, N_cards)
                  IF (k .ne. 0) THEN
                     WRITE (*,*) 'Error reading bitmap file'
                     STOP 1
                  ENDIF
                  CALL TBLOR(TABLE2,TABLE3,TABLE2,N_Cards)
               ENDIF
            enddo
            CALL TBLAND(TABLE1,TABLE2,TABLE1,N_Cards)
         endif
      ENDDO
C process allowed elements
      if (mode .ne. 0) then
         DO I=1,104
            I1 = AELEMS(I)
            if (I1 .ge. 1 .and. I1 .le. 104) then
               allowed(i1) = .true.
            elseif (I1 .le.-1 .and. I1 .ge. -18) then
               do j=1,16
                  i2 = specsym(j,-I1)
                  if (i2 .gt. 0) allowed(i2) = .true.
               enddo
            ENDIF
         enddo
         DO I=1,104
            IF (.NOT. allowed(i)) THEN
               k = ps_readbitlib(i, table2, N_cards)
               CALL TBLNOT(TABLE2,TABLE2,N_Cards)
               CALL TBLAND(TABLE1,TABLE2,TABLE1,N_Cards)
            ENDIF
         enddo
      ENDIF
      call ps_closebitlib()
C========================================================================
C now setup the history entry
C========================================================================
      trace = 'Elements Required='
      i1 = 0
      DO I=1,104
         if (elems(I) .ge. 1 .and. elems(i) .le. 104) then
            i1 = i1 + 1
            if (i1 .gt. 1) trace(lench(trace)+1:) = '+'
            trace(lench(trace)+1:) = elemlist(elems(i))
         elseif (elems(I) .le. -1 .and. elems(i) .ge. -18) then
            i1 = i1 + 1
            if (i1 .gt. 1) trace(lench(trace)+1:) = '+'
            trace(lench(trace)+1:) = '['
            i2 = 0
            do j=1,16
               if (specsym(j,-elems(i)) .gt. 0) then
                  i2 = i2 + 1
                  if (i2 .gt. 1) trace(lench(trace)+1:) = ','
                  trace(lench(trace)+1:) =elemlist(specsym(j,-elems(i)))
               endif
            enddo
            trace(lench(trace)+1:) = ']'
         endif
      enddo
      if (mode .eq. 0) then
         trace(lench(trace)+1:) = '; Allowed=all'
      ELSE
         i2 = 0
         trace(lench(trace)+1:) = '; Allowed='
         DO I=1,104
            I1 = AELEMS(I)
            if (I1 .ge. 1 .and. I1 .le. 104) then
               i2 = i2 + 1
               if (i2 .gt. 1) trace(lench(trace)+1:) = ','
               trace(lench(trace)+1:) = elemlist(i1)
            elseif (I1 .le.-1 .and. I1 .ge. -18) then
               i1 = -i1
               do j=1,16
                  if (specsym(j,i1) .gt. 0) then
                     i2 = i2 + 1
                     if (i2 .gt. 1) trace(lench(trace)+1:) = ','
                     trace(lench(trace)+1:) = elemlist(specsym(j,i1))
                  endif
               enddo
            ENDIF
         enddo
      endif
      CALL TBLLOGIC(0, trace)
      nphase = TBLCOUNT(TABLE,N_Cards)
      WRITE(*,*) 'Found ',nphase,' entries'
      STATUS = psl_savebitmap(outfile)
      if (STATUS .ne. 0) stop 1
      WRITE (*,*) 'Writing ',outfile(:lench(outfile))
      STOP
      END

